// © 2023-2024 John Breaux
//See LICENSE.md for license
//! A [Span] is a [Range] that does not implement [Iterator]. It is a [Copy] type.
use std::{
    fmt::{Debug, Display},
    ops::{Index, Range},
};
/// A <code> [Clone] + [Copy] + [!Iterator](Iterator) </code> version of a [Range]
#[derive(Clone, Copy, Default, PartialEq, Eq, PartialOrd, Ord, Hash)]
pub struct Span<Idx> {
    pub start: Idx,
    pub end: Idx,
}
impl<Idx> From<Span<Idx>> for Range<Idx> {
    fn from(value: Span<Idx>) -> Self {
        value.start..value.end
    }
}
impl<Idx> From<Range<Idx>> for Span<Idx> {
    fn from(value: Range<Idx>) -> Self {
        Self { start: value.start, end: value.end }
    }
}
impl<T> Index<Span<usize>> for [T] {
    type Output = [T];
    fn index(&self, index: Span<usize>) -> &Self::Output {
        self.index(Range::from(index))
    }
}
impl Index<Span<usize>> for str {
    type Output = str;
    fn index(&self, index: Span<usize>) -> &Self::Output {
        self.index(Range::from(index))
    }
}
impl<Idx: Debug> Debug for Span<Idx> {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "{:?}..{:?}", self.start, self.end)
    }
}
impl<Idx: Display> Display for Span<Idx> {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "{}..{}", self.start, self.end)
    }
}
